// c/c++
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <algorithm>
#include <time.h>


//root
#include "TH1F.h"
#include "TH2F.h"
#include "TFile.h"


//numerical recipes



//ranlux
#include "ranlxs.h"
#include "ranlxd.h"

void ranlxs(float r[], int n);
void ranlxd(double r[], int n);

void rlxs_init(int level, int seed);
void rlxd_init(int level, int seed);


//mz (general)
#include "mzparameters.h"
#include "mzfunctions.h"

//mz for sigma_leplep_sigma_pipi_vandewi
#include "mzfunctions_sigma_leplep_sigma_pipi_vandewi.h"

//mz for FF (only to test)
#include "mzfunctions_FF_models.h"

//mz for pp_to_leplep_vandewi
#include "mzfunctions_pp_to_leplep_vandewi.h"

//mz for pp_to_pipi_vandewi
#include "mzfunctions_pp_to_pipi_vandewi.h"


using namespace std;


int main()
{

  clock_t t_init = clock();
  double time;

  string dummy;
  int particle_flag;       // particle flag: 0=electron, 1=muon, 2=tau
  double P;                // lab frame: pbar momentum
  double cos_theta_min;    // cos(theta*) minimum, theta*=angle(lep-,pbar) CMS frame
  double cos_theta_max;    // cos(theta*) maximum
  int Nbins;               // #bins
  int FF_flag;             // form factor flag
  double R;                // ratio GE/GM
  double lumi;             // luminosity
  int seed;                // RANLUX seed
  int N;                   // number of MC points for integration
  string pathout;          // path to output
  string outfile;          // name of output file

  string fullpath_out;     // full path to root output file with histos

  double M=mp;             //proton mass
  double E;                //lab frame: pbar energy
  double S;                //center-of-mass energy squared of pbarpsystem
  double q2;               //q2

  double Re_F1, Im_F1;
  double Re_F2, Im_F2;
  double Re_GE, Im_GE;
  double Re_GM, Im_GM;
  double GE_mod, GM_mod;

  double sigma_lepton;    //integrated lepton cross section (analytical)
  double sigma_pion_MC;      //integrated pion cross section (MC)
  double delta_sigma_pion_MC;  //stat. error in integrated pion cross section (MC)

  double pb_to_fb=1000.0;         //conversion pb->fb
  double mub_to_fb=1000000000.0;  //conversion micro b -> fb

  double Nleptons;           //expected #signal events
  double Npions;             //expected #background events

  double ratio_sigma;   // (integrated sigma pion)/(10^6 * integrated sigma lepton)


  //read infile
  ifstream inFile;
  inFile.open("mzgen.in");
  inFile
    >> particle_flag >> dummy >> dummy >> dummy >> dummy >> dummy
    >> P >> dummy >> dummy >> dummy >> dummy
    >> cos_theta_min >> dummy >> dummy >> dummy
    >> cos_theta_max >> dummy >> dummy >> dummy
    >> Nbins >> dummy >> dummy
    >> FF_flag >> dummy >> dummy >> dummy >> dummy >> dummy >> dummy
    >> R >> dummy >> dummy >> dummy
    >> lumi  >> dummy >> dummy >> dummy
    >> seed >> dummy >> dummy >> dummy >> dummy
    >> N  >> dummy >> dummy >> dummy >> dummy
    >> pathout >> dummy >> dummy
    >> outfile >> dummy >> dummy;    

  inFile.close();


  printf("\n");
  printf("------------------------------------\n");
  printf("   Reading infile\n");
  printf("------------------------------------\n\n");
  printf("particle flag:               %d \n",particle_flag);
  printf("momentum pbar:               %4.2f GeV \n",P);
  printf("cos(theta*) min:             %4.2f \n",cos_theta_min);
  printf("cos(theta*) max:             %4.2f \n",cos_theta_max);
  printf("number of bins:              %d \n", Nbins);
  printf("FF flag:                     %d \n",FF_flag);
  printf("ratio GE/GM:                 %4.2f \n",R);
  printf("luminosity:                  %4.2f fb^-1 \n",lumi);
  printf("RANLUX seed:                 %d \n",seed);
  printf("number of MC points:         %d \n",N);
  cout << "pathout:                     " + pathout << endl;
  cout << "outfile:                     " + outfile << endl;

  printf("\n\n\n\n");

  fullpath_out  = pathout + "/" + outfile + ".root";


   //initializations
   mz_sigma_leplep_sigma_pipi_vandewi_init(particle_flag,P,cos_theta_min,cos_theta_max,FF_flag,seed);


   //declaration of arrays with dimension depending on #bins
   double cos_theta_array[Nbins+1];                                
   double epsilon=(cos_theta_max-cos_theta_min)/(double)(Nbins);  //bin width
   for(int i=0; i<(Nbins+1); i++){
     cos_theta_array[i]=cos_theta_min+(double)(i)*epsilon;
   }

   double sigma_lepton_array[Nbins];
   double sigma_pion_MC_array[Nbins];
   double delta_sigma_pion_MC_array[Nbins];
   double ratio_sigma_array[Nbins];

   double sum_sigma_lepton;
   double sum_sigma_pion_MC;
   double sum_sigma_pion_MC_error2;
   double sum_sigma_pion_MC_error;


   //total cross section (GE, GM from FF model)
   E=pow((P*P+M*M),0.5);
   S=2.0*M*(M+E);
   q2=S;

   //if(FF_flag==0) mz_FF_pQCD(q2, Re_GE, Im_GE, Re_GM, Im_GM, Re_F1, Im_F1, Re_F2, Im_F2);
   //if(FF_flag==1) mz_FF_vmd_mod(q2, Re_GE, Im_GE, Re_GM, Im_GM, Re_F1, Im_F1, Re_F2, Im_F2);
   //if(FF_flag==2) mz_FF_Tomasi(q2, Re_GE, Im_GE, Re_GM, Im_GM, Re_F1, Im_F1, Re_F2, Im_F2);
   mz_FF_model(FF_flag, q2, Re_GE, Im_GE, Re_GM, Im_GM, Re_F1, Im_F1, Re_F2, Im_F2);

   GE_mod=mzcomplexmod(Re_GE, Im_GE);
   GM_mod=mzcomplexmod(Re_GM, Im_GM);

   sigma_lepton = mz_pp_to_leplep_vandewi_sigma_integral(particle_flag,P,GE_mod,GM_mod,cos_theta_min,cos_theta_max);
   Nleptons = sigma_lepton*pb_to_fb*lumi;

   mz_pp_to_pipi_sigma_integral_MC(P,cos_theta_min,cos_theta_max,N,sigma_pion_MC,delta_sigma_pion_MC);
   Npions = sigma_pion_MC*mub_to_fb*lumi;

   ratio_sigma  = sigma_pion_MC/sigma_lepton;

   printf("TOTAL CROSS SECTION\n");
   printf("===================\n");
   printf("\n");

   printf("%3.2f < cos(theta*) < %3.2f \n", cos_theta_min, cos_theta_max);
   printf("\n");

   printf("GE, GM from FF model \n");
   printf("\n");

   printf("q2 = %4.2f GeV^2 \n", q2);
   printf("\n");

   printf("GE = %6.4f \n", GE_mod);
   printf("GM = %6.4f \n", GM_mod);
   printf("\n");

   printf(" => GE/GM = %6.4f \n", GE_mod/GM_mod);
   printf("\n");

   printf("sigma(e+e-)   = %12.8f                            [pb] \n", sigma_lepton);
   printf("sigma(pi+pi-) = %12.8f  +/- %12.8f (%4.2f %%) [micro b] \n", sigma_pion_MC, delta_sigma_pion_MC, delta_sigma_pion_MC/sigma_pion_MC*100.0);
   printf("\n");

   printf("N(e+e-)   = %12.0f \n", Nleptons);
   printf("N(pi+pi-) = %12.0f \n", Npions);
   printf("\n");

   printf("sigma(pi+pi-)/sigma(e+e-)*10^{-6}  = %12.8f \n", ratio_sigma);
   printf("\n\n\n\n");



   //differential cross section


   for(int i=0; i<Nbins; i++){
     sigma_lepton_array[i]=mz_pp_to_leplep_vandewi_sigma_integral(particle_flag,P,GE_mod,GM_mod,cos_theta_array[i],cos_theta_array[i+1]);
     mz_pp_to_pipi_sigma_integral_MC(P,cos_theta_array[i],cos_theta_array[i+1],N,sigma_pion_MC_array[i],delta_sigma_pion_MC_array[i]);
     ratio_sigma_array[i]=sigma_pion_MC_array[i]/sigma_lepton_array[i];
   }

   sum_sigma_lepton=0.0;
   sum_sigma_pion_MC=0.0;
   sum_sigma_pion_MC_error2=0.0;
   for(int i=0; i<Nbins; i++){
     sum_sigma_lepton  = sum_sigma_lepton + sigma_lepton_array[i];
     sum_sigma_pion_MC = sum_sigma_pion_MC + sigma_pion_MC_array[i];
     sum_sigma_pion_MC_error2 = sum_sigma_pion_MC_error2 + delta_sigma_pion_MC_array[i]*delta_sigma_pion_MC_array[i]; 
   }
   sum_sigma_pion_MC_error=pow(sum_sigma_pion_MC_error2,0.5);


   printf("BIN-INTEGRATED CROSS SECTION\n");
   printf("============================\n");
   printf("\n");

   printf("%3.2f < cos(theta*) < %3.2f,    #bins = %d    =>   cos(theta*) bin width:  a = %3.2f \n", cos_theta_min, cos_theta_max, Nbins, epsilon);
   printf("\n");

   printf("GE, GM from FF model \n");
   printf("\n");

   printf("cos(theta*) bin     sigma(e+e-) [pb]     sigma(pi+pi-) [micro b]      error [micro b]     error [%%]         ratio*10^{-6} \n");
   printf("---------------     ----------------     -----------------------      ---------------     ----------        ------------- \n");
   for(int i=1; i<(Nbins+1); i++){
     printf("%3.2f\t %3.2f\t     %12.8f            %12.8f             %12.8f      %12.8f        %12.8f\n", cos_theta_array[i-1], cos_theta_array[i], sigma_lepton_array[i-1], sigma_pion_MC_array[i-1], delta_sigma_pion_MC_array[i-1], delta_sigma_pion_MC_array[i-1]/sigma_pion_MC_array[i-1]*100.0, ratio_sigma_array[i-1]);
   }

   printf("\n\n");

   printf("sum_{i} sigma(e+e-)[i]   = %12.8f                             [pb] \n", sum_sigma_lepton);
   printf("sum_{i} sigma(pi+pi-)[i] = %12.8f   +/- %12.8f (%4.2f %%) [micro b] \n", sum_sigma_pion_MC, sum_sigma_pion_MC_error, sum_sigma_pion_MC_error/sum_sigma_pion_MC*100.0);
 
   printf("\n\n");

   printf("cos(theta*) bin     sigma(e+e-)/a [pb]     sigma(pi+pi-)/a [micro b]      error [micro b]     error [%%]         ratio*10^{-6} \n");
   printf("---------------     ----------------       -------------------------      ---------------     ----------        ------------- \n");
   for(int i=1; i<(Nbins+1); i++){
     printf("%3.2f\t %3.2f\t     %12.8f                %12.8f             %12.8f      %12.8f        %12.8f\n", cos_theta_array[i-1], cos_theta_array[i], sigma_lepton_array[i-1]/epsilon, sigma_pion_MC_array[i-1]/epsilon, delta_sigma_pion_MC_array[i-1]/epsilon, delta_sigma_pion_MC_array[i-1]/sigma_pion_MC_array[i-1]*100.0, ratio_sigma_array[i-1]);
   }
  
   printf("\n\n\n\n");


   //histos...

   int NNbins=5000;
   double x;
   double y;

  //dsigma(e+e-)/dcos(theta*)
  TH1F *h0 = new TH1F("h0","dsigma(e+e-)/dcos(theta*), GE and GM from FF model",NNbins,cos_theta_min,cos_theta_max);
  h0->GetXaxis()->SetTitle("cos(theta*)");
  h0->GetYaxis()->SetTitle("dsigma/dcos(theta*) [pb]");

  //dsigma(pi+pi-)/dcos(theta*)
  TH1F *h1 = new TH1F("h1","dsigma(pi+pi-)/dcos(theta*)",NNbins,cos_theta_min,cos_theta_max);
  h1->GetXaxis()->SetTitle("cos(theta*)");
  h1->GetYaxis()->SetTitle("dsigma/dcos(theta*) [micro b]");

  //dsigma(pi+pi-)/dsigma(e+e-)*10^{-6}
  TH1F *h2 = new TH1F("h2","dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}, GE and GM from FF model",NNbins,cos_theta_min,cos_theta_max);
  h2->GetXaxis()->SetTitle("cos(theta*)");
  h2->GetYaxis()->SetTitle("dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}");

  
  for(int i=0; i<NNbins; i++){
    x = cos_theta_min +(cos_theta_max-cos_theta_min)/(double)(NNbins)*i + 0.5*(cos_theta_max-cos_theta_min)/(double)(NNbins);

    y=mz_pp_to_leplep_vandewi_sigma(particle_flag,P,GE_mod,GM_mod,x);
    h0->SetBinContent(i+1,y);

    y=mz_pp_to_pipi_sigma_dcostheta(P,x);
    h1->SetBinContent(i+1,y);

    y=y/mz_pp_to_leplep_vandewi_sigma(particle_flag,P,GE_mod,GM_mod,x);
    h2->SetBinContent(i+1,y);
  }


  //sigma(e+e-)[i]
  TH1F *h10 = new TH1F("h10","sigma(e+e-)[i], GE and GM from FF model",Nbins,cos_theta_min,cos_theta_max);
  h10->GetXaxis()->SetTitle("cos(theta*)");
  h10->GetYaxis()->SetTitle("sigma [pb]");

  //sigma(pi+pi-)[i]
  TH1F *h11 = new TH1F("h11","sigma(pi+pi-)[i]",Nbins,cos_theta_min,cos_theta_max);
  h11->GetXaxis()->SetTitle("cos(theta*)");
  h11->GetYaxis()->SetTitle("sigma [micro b]");

  //sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^-6
  TH1F *h12 = new TH1F("h12","sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^{-6}, GE and GM from FF model",Nbins,cos_theta_min,cos_theta_max);
  h12->GetXaxis()->SetTitle("cos(theta*)");
  h12->GetYaxis()->SetTitle("sigma(pi+pi-)/sigma(e+e-)*10^{-6}");

  //sigma(e+e-)[i]/a
  TH1F *h14 = new TH1F("h14","sigma(e+e-)[i]/a, a=bin width, GE and GM from FF model",Nbins,cos_theta_min,cos_theta_max);
  h14->GetXaxis()->SetTitle("cos(theta*)");
  h14->GetYaxis()->SetTitle("sigma/a [pb]");

  //sigma(pi+pi-)[i]/a
  TH1F *h15 = new TH1F("h15","sigma(pi+pi-)[i]/a, a=bin width",Nbins,cos_theta_min,cos_theta_max);
  h15->GetXaxis()->SetTitle("cos(theta*)");
  h15->GetYaxis()->SetTitle("sigma/a [micro b]");

 
  for(int i=0; i<Nbins; i++){
    h10->SetBinContent(i+1,sigma_lepton_array[i]);
    h11->SetBinContent(i+1,sigma_pion_MC_array[i]);
    h12->SetBinContent(i+1,sigma_pion_MC_array[i]/sigma_lepton_array[i]);
    h14->SetBinContent(i+1,sigma_lepton_array[i]/epsilon);
    h15->SetBinContent(i+1,sigma_pion_MC_array[i]/epsilon);
  }



  //RESET GE, RECALCULATE LEPTON CROSS SECTION
  GE_mod=R*GM_mod;

  sigma_lepton = mz_pp_to_leplep_vandewi_sigma_integral(particle_flag,P,GE_mod,GM_mod,cos_theta_min,cos_theta_max);
  Nleptons = sigma_lepton*pb_to_fb*lumi;

  ratio_sigma  = sigma_pion_MC/sigma_lepton;


   printf("TOTAL CROSS SECTION\n");
   printf("===================\n");
   printf("\n");

   printf("%3.2f < cos(theta*) < %3.2f \n", cos_theta_min, cos_theta_max);
   printf("\n");

   printf("GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
   printf("\n");

   printf("q2 = %4.2f GeV^2 \n", q2);
   printf("\n");

   printf("GE = %6.4f \n", GE_mod);
   printf("GM = %6.4f \n", GM_mod);
   printf("\n");

   printf(" => GE/GM = %6.4f \n", GE_mod/GM_mod);
   printf("\n");

   printf("sigma(e+e-)   = %12.8f                            [pb] \n", sigma_lepton);
   printf("sigma(pi+pi-) = %12.8f  +/- %12.8f (%4.2f %%) [micro b] \n", sigma_pion_MC, delta_sigma_pion_MC, delta_sigma_pion_MC/sigma_pion_MC*100.0);
   printf("\n");

   printf("N(e+e-)   = %12.0f \n", Nleptons);
   printf("N(pi+pi-) = %12.0f \n", Npions);
   printf("\n");

   printf("sigma(pi+pi-)/sigma(e+e-)*10^{-6}  = %12.8f \n", ratio_sigma);
   printf("\n\n\n\n");


   //differential cross section


   for(int i=0; i<Nbins; i++){
     sigma_lepton_array[i]=mz_pp_to_leplep_vandewi_sigma_integral(particle_flag,P,GE_mod,GM_mod,cos_theta_array[i],cos_theta_array[i+1]);
     ratio_sigma_array[i]=sigma_pion_MC_array[i]/sigma_lepton_array[i];
   }

   sum_sigma_lepton=0.0;
   for(int i=0; i<Nbins; i++){
     sum_sigma_lepton  = sum_sigma_lepton + sigma_lepton_array[i]; 
   }


   printf("BIN-INTEGRATED CROSS SECTION\n");
   printf("============================\n");
   printf("\n");

   printf("%3.2f < cos(theta*) < %3.2f,    #bins = %d    =>   cos(theta*) bin width:  a = %3.2f \n", cos_theta_min, cos_theta_max, Nbins, epsilon);
   printf("\n");

   printf("GM from FF model, GE=R*GM, R  user-defined ratio GE/GM \n");
   printf("\n");

   printf("cos(theta*) bin     sigma(e+e-) [pb]     sigma(pi+pi-) [micro b]      error [micro b]     error [%%]         ratio*10^{-6} \n");
   printf("---------------     ----------------     -----------------------      ---------------     ----------        ------------- \n");
   for(int i=1; i<(Nbins+1); i++){
     printf("%3.2f\t %3.2f\t     %12.8f            %12.8f             %12.8f      %12.8f        %12.8f\n", cos_theta_array[i-1], cos_theta_array[i], sigma_lepton_array[i-1], sigma_pion_MC_array[i-1], delta_sigma_pion_MC_array[i-1], delta_sigma_pion_MC_array[i-1]/sigma_pion_MC_array[i-1]*100.0, ratio_sigma_array[i-1]);
   }

   printf("\n\n");

   printf("sum_{i} sigma(e+e-)[i]   = %12.8f                             [pb] \n", sum_sigma_lepton);
   printf("sum_{i} sigma(pi+pi-)[i] = %12.8f   +/- %12.8f (%4.2f %%) [micro b] \n", sum_sigma_pion_MC, sum_sigma_pion_MC_error, sum_sigma_pion_MC_error/sum_sigma_pion_MC*100.0);
 
   printf("\n\n");

   printf("cos(theta*) bin     sigma(e+e-)/a [pb]     sigma(pi+pi-)/a [micro b]      error [micro b]     error [%%]         ratio*10^{-6} \n");
   printf("---------------     ----------------       -------------------------      ---------------     ----------        ------------- \n");
   for(int i=1; i<(Nbins+1); i++){
     printf("%3.2f\t %3.2f\t     %12.8f                %12.8f             %12.8f      %12.8f        %12.8f\n", cos_theta_array[i-1], cos_theta_array[i], sigma_lepton_array[i-1]/epsilon, sigma_pion_MC_array[i-1]/epsilon, delta_sigma_pion_MC_array[i-1]/epsilon, delta_sigma_pion_MC_array[i-1]/sigma_pion_MC_array[i-1]*100.0, ratio_sigma_array[i-1]);
   }
  
   printf("\n\n\n\n");



  //histos...

  //dsigma(e+e-)/dcos(theta*)
  TH1F *h100 = new TH1F("h100","dsigma(e+e-)/dcos(theta*), GM from FF model, GE=R*GM, R user-defined ratio GE/GM",NNbins,cos_theta_min,cos_theta_max);
  h100->GetXaxis()->SetTitle("cos(theta*)");
  h100->GetYaxis()->SetTitle("dsigma/dcos(theta*) [pb]");

  //dsigma(pi+pi-)/dsigma(e+e-)*10^{-6}
  TH1F *h102 = new TH1F("h102","dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}",NNbins,cos_theta_min,cos_theta_max);
  h102->GetXaxis()->SetTitle("cos(theta*)");
  h102->GetYaxis()->SetTitle("dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}");

  
  for(int i=0; i<NNbins; i++){
    x = cos_theta_min +(cos_theta_max-cos_theta_min)/(double)(NNbins)*i + 0.5*(cos_theta_max-cos_theta_min)/(double)(NNbins);

    y=mz_pp_to_leplep_vandewi_sigma(particle_flag,P,GE_mod,GM_mod,x);
    h100->SetBinContent(i+1,y);
 
    y=mz_pp_to_pipi_sigma_dcostheta(P,x)/y;
    h102->SetBinContent(i+1,y);
  }



  //sigma(e+e-)[i]
  TH1F *h110 = new TH1F("h110","sigma(e+e-)[i],GM from FF model, GE=R*GM, R user-defined ratio GE/GM",Nbins,cos_theta_min,cos_theta_max);
  h110->GetXaxis()->SetTitle("cos(theta*)");
  h110->GetYaxis()->SetTitle("sigma [pb]");

  //sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^-6
  TH1F *h112 = new TH1F("h112","sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^{-6}, GM from FF model, GE=R*GM, R user-defined ratio GE/GM",Nbins,cos_theta_min,cos_theta_max);
  h112->GetXaxis()->SetTitle("cos(theta*)");
  h112->GetYaxis()->SetTitle("sigma(pi+pi-)/sigma(e+e-)*10^{-6}");

  //sigma(e+e-)[i]/a
  TH1F *h114 = new TH1F("h114","sigma(e+e-)[i]/a, a=bin width, GM from FF model, GE=R*GM, R user-defined ratio GE/GM",Nbins,cos_theta_min,cos_theta_max);
  h114->GetXaxis()->SetTitle("cos(theta*)");
  h114->GetYaxis()->SetTitle("sigma/a [pb]");


  for(int i=0; i<Nbins; i++){
    h110->SetBinContent(i+1,sigma_lepton_array[i]);
    h112->SetBinContent(i+1,sigma_pion_MC_array[i]/sigma_lepton_array[i]);
    h114->SetBinContent(i+1,sigma_lepton_array[i]/epsilon);
  }



  /**
   * open file, write histos to file and close file
   */


  //open
  TFile outputFile(fullpath_out.c_str(),"recreate");


  //write histos to file...

  h0->Write();
  h0->Draw();

  h1->Write();
  h1->Draw();

  h2->Write();
  h2->Draw();



  h10->Write();
  h10->Draw();

  h11->Write();
  h11->Draw();

  h12->Write();
  h12->Draw();

  h14->Write();
  h14->Draw();

  h15->Write();
  h15->Draw();



  h100->Write();
  h100->Draw();

  h102->Write();
  h102->Draw();



  h110->Write();
  h110->Draw();

  h112->Write();
  h112->Draw();

  h114->Write();
  h114->Draw();


  //close file
  outputFile.Close();
  printf("\n\n\n\n");


  printf("Writing histos to file: \n");
  cout << "     " << fullpath_out << "   (root file)" << endl;
  printf("\n\n");


  printf("HISTOGRAM DESCRIPTION\n");
  printf("=====================\n");
  printf("\n");

  printf("h0  -> dsigma(e+e-)/dcos(theta*), GE and GM from FF model \n");
  printf("h1  -> dsigma(pi+pi-)/dcos(theta*) \n");
  printf("h2  -> dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}, GE and GM from FF model  \n");
  printf("\n");  

  printf("h10 -> sigma(e+e-)[i], GE and GM from FF model \n");
  printf("h11 -> sigma(pi+pi-)[i] \n");
  printf("h12 -> sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^{-6}, GE and GM from FF model \n");
  printf("h14 -> sigma(e+e-)[i]/a, a=bin width, GE and GM from FF model \n");
  printf("h15 -> sigma(pi+pi-)[i]/a, a=bin width \n");
  printf("\n");

  printf("h100  -> dsigma(e+e-)/dcos(theta*), GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
  printf("h102  -> dsigma(pi+pi-)/dsigma(e^+e^-)*10^{-6}, GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
  printf("\n");  


  printf("h110 -> sigma(e+e-)[i], GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
  printf("h112 -> sigma(pi+pi-)[i]/sigma(e+e-)[i]*10^{-6}, GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
  printf("h114 -> sigma(e+e-)[i]/a, a=bin width, GM from FF model, GE=R*GM, R user-defined ratio GE/GM \n");
  printf("\n");


  printf("\n\n");

  time=(double)(clock() - t_init)/CLOCKS_PER_SEC;
  printf("Execution time... %.2fs \n",time);
  printf("\n\n");



  return 0;
}
